import pandas as pd
import seaborn as sns

def run_forecasts()

def generate_dashboard():
    df = pd.read_csv("raw_yield_data.csv")

    #region Add columns
    df = run_forecasts(df)
    df['Week'] =

    #endregion